<?PHP
/*JJserver php-cli控制函数*/
/*
	配置文件变量：
	php.ini=>php.jj {	
		ext 扩展目录 [$phpextdir] 
	}
	my.ini=>my.jj {	
		mysql的端口 [$mysqlport] 3306  
	}
	nginx.conf=>nginx.jj {	 
		网站根目录 [$nginxroot]  
		网站端口号 [$httpport] 默认：80
	}
	redis.conf=>redis.jj {	 
		redis数据库存储目录 [$redisdatadir] ---临时录即可   
		redis端口 [$redisport]  默认：6379  redis.conf配置文件也放置到临时目录 
	}
*/
class JJserver 
{	
	public $ServerRoot;//网站根目录
	public $JJServerRoot;//程序放置的./JJServer目录的绝对目录
	public $NginxDir;//nginx程序目录 ./JJserver/bin/nginx-1.5.1 绝对目录
	public $NginxConfDir;//nginx配置文件目录 ./JJserver/bin/nginx-1.5.1/conf 绝对目录
	public $NginxBinDir;//nginx可执行文件目录 ./JJserver/bin/nginx-1.5.1 绝对目录
	public $PhpDir;
	public $PhpConfDir;
	public $PhpBinDir;
	public $MysqlDir;
	public $MysqlConfDir;
	public $MysqlBinDir;
	public $RedisDir;
	public $RedisConfDir;
	public $RedisBinDir;
	public $Http_Port;
	public $Mysql_Port;
	public $Redis_Port;
	public $SystemTmpDir; //系统临时文件目录
	/*传入配置文件数组*/
	public function __construct($JJserverConf)
	{
		
		$tmp = realpath(preg_replace('/php-tools/','',__dir__));
		$this->JJServerRoot = $tmp;
		
		$this->NginxDir = $tmp.'/nginx-1.5.1';
		$this->NginxConfDir = $tmp.'/nginx-1.5.1/conf';
		$this->NginxBinDir = $this->NginxDir;
		
		$this->PhpDir = $tmp.'/php-5.3.26';
		$this->PhpConfDir = $this->PhpDir;
		$this->PhpBinDir = $this->PhpDir;
		
		$this->MysqlDir = $tmp.'/mysql-5.5.32';
		$this->MysqlConfDir = $this->MysqlDir;
		$this->MysqlBinDir = $this->MysqlDir.'/bin';
		
		$this->RedisDir =  $tmp.'/redis-2.6.12';
		$this->RedisConfDir =  $this->RedisDir;
		$this->RedisBinDir =  $this->RedisDir;
		
		$this->SystemTmpDir = sys_get_temp_dir();
		if(stristr(trim($this->SystemTmpDir),' '))
		{
			Display('您的系统临时文件目录名称中存在空格，请修改修改系统临时目录');
			die('');
		}
		
		$tmps = realpath(__dir__.DIRECTORY_SEPARATOR.'../../wwwroot');
		$this->ServerRoot = $tmps;
		if(trim($JJserverConf['Server_Root'])) {
			$this->ServerRoot = trim($JJserverConf['Server_Root']);
		}
		
		$this->Http_Port = 80;
		$this->Mysql_Port = 3306;
		$this->Redis_Port = 6379;

		if(trim($JJserverConf['Http_Port'])) {
			$this->Http_Port = trim($JJserverConf['Http_Port']);
		}
		if(trim($JJserverConf['Mysql_Port'])) {
			$this->Mysql_Port = trim($JJserverConf['Mysql_Port']);
		}
		if(trim($JJserverConf['Redis_Port'])) {
			$this->Redis_Port = trim($JJserverConf['Redis_Port']);
		}
		
		/*初始化检测各个端口占用情况*/
		Display('系统初始化中...');
		$this->CheckConfPort();
		Display('系统正在检测配置文件完整性 请稍后...');
		$this->ConfConfig();
	}
	/*系统启动自动检测配置文件中的各个端口占用情况*/
	private function CheckConfPort() {
		Display('正在检测nginx服务端口:'.$this->Http_Port);
		$i = 0 ;
		while(CheckPort($this->Http_Port) && $i<45)
		{
			if($i<1) {
				Display($this->Http_Port.'端口被占用，系统正在尝试关闭JJserver可能存在的nginx进程后重试');
				$this->clean_nginx();
			}else {
				Display($this->Http_Port."端口被占用,若想继使用该端口,请立即关闭占用".$this->Http_Port."端口的其他软件");
				Display('正在第'.$i.'次尝试为JJserver中的nginx启用'.$this->Http_Port.'端口.....');
			}
			sleep(1) ;
			$i++ ;
		}
		if($i >= 45)	
		{
			Display($this->Http_Port.'端口被占用，请在JJserver配置文件JJserver.conf.php中更改');
			die('');
		}
		Success();
		/*----------------*/
		Display('正在检测mysql服务端口:'.$this->Mysql_Port);
		$i = 0 ;
		while(CheckPort($this->Mysql_Port) && $i<45)
		{
			if($i<1) {
				Display($this->Mysql_Port."端口被占用，系统正在尝试关闭JJserver可能存在的mysql进程后重试\n");
				$this->stop_mysql();
			}else {
				Display('正在第'.($i+1).'次尝试为JJserver中的Mysql启用'.$this->Mysql_Port.'端口.....');
			}
			sleep(1) ;
			$i++ ;
		}
		if($i >= 45)	
		{
			Display($this->Mysql_Port.'端口被占用，请在JJserver配置文件JJserver.conf.php中更改');
			die('');
		}
		Success();
		/*----------------*/
		Display('正在检测redis服务端口:'.$this->Redis_Port);
		$i = 0 ;
		while(CheckPort($this->Redis_Port) && $i<45)
		{
			Display('正在第'.($i+1).'次尝试为Redis启用'.$this->Redis_Port.'端口.....');
			sleep(1) ;
			$i++ ;
		}
		if($i >= 45)	
		{
			Display($this->Redis_Port.'端口被占用，请在JJserver配置文件JJserver.conf.php中更改');
			die('');
		}
		Success();
	}
	/*各个软件配置文件修改检测*/
	public function ConfConfig() {
		/*nginx*/
		$nginxconf = $this->NginxConfDir.'/nginx.jj';
		if(!file_exists($nginxconf)) {
			Display("JJserver中nginx的核心配置文件nginx.jj丢失....Error\nExit..");
			die(''); 
		}
		Display("正在检测nginx配置文件");
		$nginxconf = file_get_contents($nginxconf);
		$nginxconf = str_replace(array('[$nginxroot]','[$httpport]'),array($this->ServerRoot,$this->Http_Port),$nginxconf);
		file_put_contents($this->NginxConfDir.'/nginx.conf',$nginxconf,LOCK_EX);
		Success();
		/*php*/
		$phpconf = $this->PhpConfDir.'/php.jj';
		if(!file_exists($phpconf)) {
			Display("JJserver中php的核心配置文件nginx.jj丢失....Error\nExit..");
			die(''); 
		}
		Display("正在检测php配置文件");
		$phpconf = file_get_contents($phpconf);
		$phpconf = str_replace(array('[$phpextdir]'),array($this->PhpConfDir),$phpconf);
		file_put_contents($this->PhpConfDir.'/php.ini',$phpconf,LOCK_EX);
		Success();
		/*mysql*/
		$mysqlconf = $this->MysqlConfDir.'/my.jj';
		if(!file_exists($mysqlconf)) {
			Display("JJserver中mysql的核心配置文件my.jj丢失....Error\nExit..");
			die(''); 
		}
		Display("正在检测mysql配置文件");
		$mysqlconf = file_get_contents($mysqlconf);
		$mysqlconf = str_replace(array('[$mysqlport]'),array($this->Mysql_Port),$mysqlconf);
		file_put_contents($this->MysqlConfDir.'/my.ini',$mysqlconf,LOCK_EX);
		Success();
		/*php*/
		$redisconf = $this->RedisConfDir.'/redis.jj';
		if(!file_exists($redisconf)) {
			Display("JJserver中redis的核心配置文件nginx.jj丢失....Error\nExit..");
			die(''); 
		}
		Display("正在检测redis配置文件");
		$redisconf = file_get_contents($redisconf);
		$redisconf = str_replace(array('[$redisdatadir]','[$redisport]'),array($this->SystemTmpDir,$this->Redis_Port),$redisconf);
		file_put_contents($this->RedisConfDir.'/redis.conf',$redisconf,LOCK_EX);
		file_put_contents($this->SystemTmpDir.'/redis.conf',$redisconf,LOCK_EX);
		Success();
	}
	/*启动mysql*/
	public function init_start_mysql() {
		Display("正在启动Mysql服务器.....");
		if(!$this->mysql_is_run_or_install(false)) {
			//mysql服务尚未安装
			Display("正在安装mysql服务，服务名称:JJserverNginxmysql");
			$this->install_mysql();
			Success();
			//启动
			Display("正在启动mysql服务");
			$this->start_mysql();
			Success();
		}else {
			if($this->mysql_is_run_or_install(true)) {
				//mysql服务已安装并且启动
				Display("系统检测到mysql服务已存在，可能是您上次的JJserver未能从容退出");
			}else {
				$this->start_mysql();
				Success();
			}
		}
	}
	/*安装mysql服务*/
	public function install_mysql() {
		run($this->MysqlBinDir.'/mysqld.exe --install JJserverNginxmysql','',$apacheresources) ;
	}
	/*卸载mysql服务*/
	public function uninstall_mysql() {
		run($this->MysqlBinDir.'/mysqld.exe --remove JJserverNginxmysql','',$apacheresources) ;
	}
	public function start_mysql() {
		$str ='net start JJserverNginxmysql';
		run($str,'',$apacheresources) ;
	}
	/*停止mysql*/
	public function stop_mysql() {
		//$str = $this->JJserver_dir.'\bin\mysql-5.1.70\bin\mysqld.exe --defaults-file='.$this->JJserver_dir.'\bin\mysql-5.1.70\my.ini --standalone';
		$str ='net stop JJserverNginxmysql';
		run($str,'',$apacheresources) ;
		return;
	}
	/*启动nginx 需要在直接所运行的文件中执行 此处仅作代码写法示例*/
	public function init_start_nginx() {
		//$this->NginxConfDir;
		$str = $this->NginxBinDir.'/nginx.exe -p '.$this->NginxDir;
		run($str,'',$apacheresources,true) ;
	}
	/*清除nginx进程*/
	public function clean_nginx() {
		$nginxpid = $this->NginxDir.'/logs/nginx.pid';
		if(file_exists($nginxpid)) {
			//nginx进程存在 关闭之
			$str = 'taskkill /f /im nginx.exe';
			run($str,'',$apacheresources) ;
			unlink($nginxpid);
		}
	}
	public function start_nginx() {
		$str = $this->NginxBinDir.'/nginx.exe -p '.$this->NginxDir;
		run($str,'',$apacheresources,true) ;
	}
	/*停止nginx*/
	public function stop_nginx() {
		$str = $this->NginxBinDir.'/nginx.exe -p '.$this->NginxDir.' -s stop';
		run($str,'',$stop,true) ;
	}
	/*重新启动nginx*/
	public function restart_nginx() {
		$str = $this->NginxBinDir.'/nginx.exe -p '.$this->NginxDir.' -s restart';
		run($str,'',$apacheresources) ;
	}
	/*检查nginx的配置是否正确*/
	public function test_nginx() {
		Display("\n");
		$str = $this->NginxBinDir.'/nginx.exe -p '.$this->NginxDir.' -t';
		run($str,'',$apacheresources) ;
		Display();
	}
	/*启动php-cgi*/
	public function init_start_php() {
		$str = $this->PhpBinDir.'\php-cgi.exe -b 127.0.0.1:9000 -c '.$this->PhpConfDir.'\php.ini';
		run($str,'',$apacheresources,true) ;
	}
	public function stop_phpcgi() {
		$str = 'taskkill /f /im php-cgi.exe';
		run($str,'',$phpcgi) ;
	}
	/*停止redis*/
	public function stop_redis() {
		run('"'.$this->RedisBinDir.'\redis-cli.exe" shutdown','',$redisstop,true);
	}
	//判断apache服务是否安装，并且可以查看apache服务是否启动。
	//$flag = true返回mysql服务是否启动
	//$flag = false返回mysql服务是否安装
	//sc qc JJserverNginxmysql  
	public function mysql_is_run_or_install($flag = true)
	{
		if($flag == true)
			$str = rerun("sc query",'',$runresources) ;
		else
			$str = rerun("sc query state= all",'',$runresources) ;
		if(strpos($str,"SERVICE_NAME: JJserverNginxmysql") === false)
			return false ;
		else
			return true ;
	}
}
/*检测端口占用情况*/
function CheckPort($port) 
{
		$str = rerun('netstat -ano | findstr ".*[0-9]:'.$port.'[^0-9].*[0-9]:[0-9].*"','',$runresources) ;
		if($str == "")
			return false ;
		else
			return true ;
}
/*模拟命令行输入模式*/
function commend() {
	echo iconv('UTF-8','GBK',"JJserver Command:");
}
/*模拟命令行输入模式*/
function Display($str='') {
	echo iconv('UTF-8','GBK',"\nJJserver Command:".$str);
}
/*成功提示信息输出*/
function Success($str='') {
	echo iconv('UTF-8','GBK',"\nJJserver Command:".$str.'Success!');
}
/*运行cmd命令*/
function run($dir,$pstr,&$runresources = null,$is_redis=false){
	echo iconv('UTF-8','GBK',$pstr);
	$runresources = proc_open($dir,array( 0 => array('pipe', 'r'), 1 => array('pipe', 'w') ),$pipesarray);
	stream_set_blocking($pipesarray[1],0);
	//sleep(1) ;
	//var_dump($pipesarray);
	fclose($pipesarray[0]) ;
	fclose($pipesarray[1]) ;
	$ret = proc_get_status($runresources);
	if($is_redis == false) 
	proc_close($runresources) ;
	return $ret ;
}
/*重新运行cmd命令*/
function rerun($dir,$pstr,&$runresources = null){
	$runresources = proc_open($dir,array( 0 => array('pipe', 'r'), 1 => array('pipe', 'w') ),$pipesarray);
	stream_set_blocking($pipesarray[1],0);
	echo iconv('UTF-8','GBK',$pstr);
	$ret = stream_get_contents($pipesarray[1]);
	fclose($pipesarray[0]) ;
	fclose($pipesarray[1]) ;
	proc_close($runresources) ;
	return $ret ;
}
/*分析用户输入的命令*/
function analyze($read) {
	if(empty($read))
		return false ;
	$command = array() ;
	$argument = preg_split("/[\s\t]/",$read) ;
	for($i=0;$i<count($argument);$i++)
	{
		$temp = $argument[$i] ;
		$temp = trim($temp) ;
		if($temp != "")
		{
			array_push($command,$temp) ;
		}
	}
	return $command ;
}
function clearScreen() {
	$temp = exec('chcp') ;
	$ttt = explode(':',$temp) ;
	$chcp = trim($ttt[1]) ;
	exec("chcp 437 >nul") ;
	exec("chcp $chcp >nul") ;
}
?>